@@ -1,6 +1,9 @@
 # pylint: disable=too-many-lines
 """Nginx Configuration"""
+import atexit
+from contextlib import ExitStack
 import logging
+import importlib.resources
 import re
 import socket
 import subprocess
@@ -8,26 +11,15 @@ import tempfile
 import time
 from typing import Any
 from typing import Callable
-from typing import Dict
 from typing import Iterable
-from typing import List
 from typing import Mapping
 from typing import Optional
 from typing import Sequence
-from typing import Set
-from typing import Text
-from typing import Tuple
-from typing import Type
 from typing import Union
+from typing import cast
 
-from certbot_nginx._internal import constants
-from certbot_nginx._internal import display_ops
-from certbot_nginx._internal import http_01
-from certbot_nginx._internal import nginxparser
-from certbot_nginx._internal import obj
-from certbot_nginx._internal import parser
-import OpenSSL
-import pkg_resources
+from cryptography.hazmat.primitives import serialization
+from cryptography.hazmat.primitives.asymmetric import rsa
 
 from acme import challenges
 from acme import crypto_util as acme_crypto_util
@@ -38,6 +30,12 @@ from certbot import util
 from certbot.compat import os
 from certbot.display import util as display_util
 from certbot.plugins import common
+from certbot_nginx._internal import constants
+from certbot_nginx._internal import display_ops
+from certbot_nginx._internal import http_01
+from certbot_nginx._internal import nginxparser
+from certbot_nginx._internal import obj
+from certbot_nginx._internal import parser
 
 NAME_RANK = 0
 START_WILDCARD_RANK = 1
@@ -116,8 +114,8 @@ class NginxConfigurator(common.Configurator):
 
         # List of vhosts configured per wildcard domain on this run.
         # used by deploy_cert() and enhance()
-        self._wildcard_vhosts: Dict[str, List[obj.VirtualHost]] = {}
-        self._wildcard_redirect_vhosts: Dict[str, List[obj.VirtualHost]] = {}
+        self._wildcard_vhosts: dict[str, list[obj.VirtualHost]] = {}
+        self._wildcard_redirect_vhosts: dict[str, list[obj.VirtualHost]] = {}
 
         # Add number of outstanding challenges
         self._chall_out = 0
@@ -146,15 +144,24 @@ class NginxConfigurator(common.Configurator):
         # many versions of OpenSSL. So we have to check both for the two different features,
         # leading to four different combinations of options.
         # For a complete history, check out https://github.com/certbot/certbot/issues/7322
+        #
+        # Technically, nginx >= 1.23.2 has session tickets off by default, and therefore
+        # no longer needs it explicitly set. But since older versions than that are still
+        # around in the oldest non-deprecated rhel, debian, and ubuntu, we will keep explicitly
+        # setting it for now to reduce complexity.
 
         use_tls13 = self.version >= (1, 13, 0)
         min_openssl_version = util.parse_loose_version('1.0.2l')
         session_tix_off = self.version >= (1, 5, 9) and self.openssl_version and\
             util.parse_loose_version(self.openssl_version) >= min_openssl_version
 
+        deprecated_conf = True
+
         if use_tls13:
             if session_tix_off:
+                # current version
                 config_filename = "options-ssl-nginx.conf"
+                deprecated_conf = False
             else:
                 config_filename = "options-ssl-nginx-tls13-session-tix-on.conf"
         else:
@@ -163,8 +170,18 @@ class NginxConfigurator(common.Configurator):
             else:
                 config_filename = "options-ssl-nginx-old.conf"
 
-        return pkg_resources.resource_filename(
-            "certbot_nginx", os.path.join("_internal", "tls_configs", config_filename))
+        if deprecated_conf:
+            logger.warning('Certbot has detected that nginx version < 1.13.0 or compiled against '
+                'openssl < 1.0.2l. Since these are deprecated, the configuration file being '
+                'installed at %s will not receive future updates. To get the latest configuration '
+                'version, update nginx.', self.mod_ssl_conf)
+
+        file_manager = ExitStack()
+        atexit.register(file_manager.close)
+        ref = (importlib.resources.files("certbot_nginx").joinpath("_internal")
+               .joinpath("tls_configs").joinpath(config_filename))
+
+        return str(file_manager.enter_context(importlib.resources.as_file(ref)))
 
     @property
     def mod_ssl_conf(self) -> str:
@@ -264,7 +281,7 @@ class NginxConfigurator(common.Configurator):
         self.save_notes += "\tssl_certificate_key %s\n" % key_path
 
     def _choose_vhosts_wildcard(self, domain: str, prefer_ssl: bool,
-                                no_ssl_filter_port: Optional[str] = None) -> List[obj.VirtualHost]:
+                                no_ssl_filter_port: Optional[str] = None) -> list[obj.VirtualHost]:
         """Prompts user to choose vhosts to install a wildcard certificate for"""
         if prefer_ssl:
             vhosts_cache = self._wildcard_vhosts
@@ -315,13 +332,13 @@ class NginxConfigurator(common.Configurator):
     #######################
     # Vhost parsing methods
     #######################
-    def _choose_vhost_single(self, target_name: str) -> List[obj.VirtualHost]:
+    def _choose_vhost_single(self, target_name: str) -> list[obj.VirtualHost]:
         matches = self._get_ranked_matches(target_name)
         vhosts = [x for x in [self._select_best_name_match(matches)] if x is not None]
         return vhosts
 
     def choose_vhosts(self, target_name: str,
-                      create_if_no_match: bool = False) -> List[obj.VirtualHost]:
+                      create_if_no_match: bool = False) -> list[obj.VirtualHost]:
         """Chooses a virtual host based on the given domain name.
 
         .. note:: This makes the vhost SSL-enabled if it isn't already. Follows
@@ -365,13 +382,14 @@ class NginxConfigurator(common.Configurator):
 
         return vhosts
 
-    def ipv6_info(self, port: str) -> Tuple[bool, bool]:
+    def ipv6_info(self, host: str, port: str) -> tuple[bool, bool]:
         """Returns tuple of booleans (ipv6_active, ipv6only_present)
         ipv6_active is true if any server block listens ipv6 address in any port
 
         ipv6only_present is true if ipv6only=on option exists in any server
         block ipv6 listen directive for the specified port.
 
+        :param str host: Host to check ipv6only=on directive for
         :param str port: Port to check ipv6only=on directive for
 
         :returns: Tuple containing information if IPv6 is enabled in the global
@@ -385,7 +403,7 @@ class NginxConfigurator(common.Configurator):
             for addr in vh.addrs:
                 if addr.ipv6:
                     ipv6_active = True
-                if addr.ipv6only and addr.get_port() == port:
+                if addr.ipv6only and addr.get_port() == port and addr.get_addr() == host:
                     ipv6only_present = True
         return ipv6_active, ipv6only_present
 
@@ -439,7 +457,7 @@ class NginxConfigurator(common.Configurator):
                                            f"block for {domain}. Set the `server_name` directive "
                                            "to use the Nginx installer.")
 
-    def _get_ranked_matches(self, target_name: str) -> List[Dict[str, Any]]:
+    def _get_ranked_matches(self, target_name: str) -> list[dict[str, Any]]:
         """Returns a ranked list of vhosts that match target_name.
         The ranking gives preference to SSL vhosts.
 
@@ -469,12 +487,12 @@ class NginxConfigurator(common.Configurator):
             # Wildcard match - need to find the longest one
             rank = matches[0]['rank']
             wildcards = [x for x in matches if x['rank'] == rank]
-            return max(wildcards, key=lambda x: len(x['name']))['vhost']
+            return cast(obj.VirtualHost, max(wildcards, key=lambda x: len(x['name']))['vhost'])
         # Exact or regex match
-        return matches[0]['vhost']
+        return cast(obj.VirtualHost, matches[0]['vhost'])
 
     def _rank_matches_by_name(self, vhost_list: Iterable[obj.VirtualHost],
-                              target_name: str) -> List[Dict[str, Any]]:
+                              target_name: str) -> list[dict[str, Any]]:
         """Returns a ranked list of vhosts from vhost_list that match target_name.
         This method should always be followed by a call to _select_best_name_match.
 
@@ -512,7 +530,7 @@ class NginxConfigurator(common.Configurator):
         return sorted(matches, key=lambda x: x['rank'])
 
     def _rank_matches_by_name_and_ssl(self, vhost_list: Iterable[obj.VirtualHost],
-                                      target_name: str) -> List[Dict[str, Any]]:
+                                      target_name: str) -> list[dict[str, Any]]:
         """Returns a ranked list of vhosts from vhost_list that match target_name.
         The ranking gives preference to SSLishness before name match level.
 
@@ -529,7 +547,7 @@ class NginxConfigurator(common.Configurator):
                 match['rank'] += NO_SSL_MODIFIER
         return sorted(matches, key=lambda x: x['rank'])
 
-    def choose_redirect_vhosts(self, target_name: str, port: str) -> List[obj.VirtualHost]:
+    def choose_redirect_vhosts(self, target_name: str, port: str) -> list[obj.VirtualHost]:
         """Chooses a single virtual host for redirect enhancement.
 
         Chooses the vhost most closely matching target_name that is
@@ -557,8 +575,8 @@ class NginxConfigurator(common.Configurator):
             vhosts = [x for x in [self._select_best_name_match(matches)]if x is not None]
         return vhosts
 
-    def choose_auth_vhosts(self, target_name: str) -> Tuple[List[obj.VirtualHost],
-                                                            List[obj.VirtualHost]]:
+    def choose_auth_vhosts(self, target_name: str) -> tuple[list[obj.VirtualHost],
+                                                            list[obj.VirtualHost]]:
         """Returns a list of HTTP and HTTPS vhosts with a server_name matching target_name.
 
         If no HTTP vhost exists, one will be cloned from the default vhost. If that fails, no HTTP
@@ -626,7 +644,7 @@ class NginxConfigurator(common.Configurator):
     def _vhost_listening_on_port_no_ssl(self, vhost: obj.VirtualHost, port: str) -> bool:
         return self._vhost_listening(vhost, port, False)
 
-    def _get_redirect_ranked_matches(self, target_name: str, port: str) -> List[Dict[str, Any]]:
+    def _get_redirect_ranked_matches(self, target_name: str, port: str) -> list[dict[str, Any]]:
         """Gets a ranked list of plaintextish port-listening vhosts matching target_name
 
         Filter all hosts for those listening on port without using ssl.
@@ -648,7 +666,7 @@ class NginxConfigurator(common.Configurator):
 
         return self._rank_matches_by_name(matching_vhosts, target_name)
 
-    def get_all_names(self) -> Set[str]:
+    def get_all_names(self) -> set[str]:
         """Returns all names found in the Nginx Configuration.
 
         :returns: All ServerNames, ServerAliases, and reverse DNS entries for
@@ -656,7 +674,7 @@ class NginxConfigurator(common.Configurator):
         :rtype: set
 
         """
-        all_names: Set[str] = set()
+        all_names: set[str] = set()
 
         for vhost in self.parser.get_vhosts():
             try:
@@ -681,23 +699,32 @@ class NginxConfigurator(common.Configurator):
                         else:
                             socket.inet_pton(socket.AF_INET, host)
                         all_names.add(socket.gethostbyaddr(host)[0])
-                    except (socket.error, socket.herror, socket.timeout):
+                    except (OSError, socket.herror, socket.timeout):
                         continue
 
         return util.get_filtered_names(all_names)
 
-    def _get_snakeoil_paths(self) -> Tuple[str, str]:
+    def _get_snakeoil_paths(self) -> tuple[str, str]:
         """Generate invalid certs that let us create ssl directives for Nginx"""
         # TODO: generate only once
         tmp_dir = os.path.join(self.config.work_dir, "snakeoil")
         le_key = crypto_util.generate_key(
-            key_size=1024, key_dir=tmp_dir, keyname="key.pem",
+            key_type='rsa', key_size=2048, key_dir=tmp_dir, keyname="key.pem",
             strict_permissions=self.config.strict_permissions)
-        key = OpenSSL.crypto.load_privatekey(
-            OpenSSL.crypto.FILETYPE_PEM, le_key.pem)
-        cert = acme_crypto_util.gen_ss_cert(key, domains=[socket.gethostname()])
-        cert_pem = OpenSSL.crypto.dump_certificate(
-            OpenSSL.crypto.FILETYPE_PEM, cert)
+        assert le_key.file is not None
+        cryptography_key = serialization.load_pem_private_key(le_key.pem, password=None)
+        assert isinstance(cryptography_key, rsa.RSAPrivateKey)
+        cert = acme_crypto_util.make_self_signed_cert(
+            cryptography_key,
+            # we used to use socket.gethostname here, but that sometimes
+            # resulted in strings over 64 characters long which would error
+            # on the validation introduced in
+            # https://github.com/pyca/cryptography/pull/11201. the ".invalid"
+            # TLD comes from RFC2606 (and was also used in the tls-sni-01
+            # challenge in early versions of the ACME spec)
+            domains=['temp-certbot-nginx.invalid']
+        )
+        cert_pem = cert.public_bytes(serialization.Encoding.PEM)
         cert_file, cert_path = util.unique_file(
             os.path.join(tmp_dir, "cert.pem"), mode="wb")
         with cert_file:
@@ -714,9 +741,16 @@ class NginxConfigurator(common.Configurator):
 
         """
         https_port = self.config.https_port
-        ipv6info = self.ipv6_info(str(https_port))
-        ipv6_block = ['']
-        ipv4_block = ['']
+        http_port = self.config.http01_port
+
+        # no addresses should have ssl turned on here
+        assert not vhost.ssl
+
+        addrs_to_insert: list[obj.Addr] = [
+            obj.Addr.fromstring(f'{addr.get_addr()}:{https_port} ssl')
+            for addr in vhost.addrs
+            if addr.get_port() == str(http_port)
+        ]
 
         # If the vhost was implicitly listening on the default Nginx port,
         # have it continue to do so.
@@ -724,31 +758,46 @@ class NginxConfigurator(common.Configurator):
             listen_block = [['\n    ', 'listen', ' ', self.DEFAULT_LISTEN_PORT]]
             self.parser.add_server_directives(vhost, listen_block)
 
-        if vhost.ipv6_enabled():
-            ipv6_block = ['\n    ',
-                          'listen',
-                          ' ',
-                          '[::]:{0}'.format(https_port),
-                          ' ',
-                          'ssl']
-            if not ipv6info[1]:
-                # ipv6only=on is absent in global config
-                ipv6_block.append(' ')
-                ipv6_block.append('ipv6only=on')
+        if not addrs_to_insert:
+            # there are no existing addresses listening on 80
+            if vhost.ipv6_enabled():
+                addrs_to_insert += [obj.Addr.fromstring(f'[::]:{https_port} ssl')]
+            if vhost.ipv4_enabled():
+                addrs_to_insert += [obj.Addr.fromstring(f'{https_port} ssl')]
 
-        if vhost.ipv4_enabled():
-            ipv4_block = ['\n    ',
-                          'listen',
-                          ' ',
-                          '{0}'.format(https_port),
-                          ' ',
-                          'ssl']
+        addr_blocks: list[list[str]] = []
+        ipv6only_set_here: set[tuple[str, str]] = set()
+        for addr in addrs_to_insert:
+            host = addr.get_addr()
+            port = addr.get_port()
+            if addr.ipv6:
+                addr_block = ['\n    ',
+                              'listen',
+                              ' ',
+                              f'{host}:{port}',
+                              ' ',
+                              'ssl']
+                ipv6only_exists = self.ipv6_info(host, port)[1]
+                if not ipv6only_exists and (host, port) not in ipv6only_set_here:
+                    addr.ipv6only = True # bookkeeping in case we switch output implementation
+                    ipv6only_set_here.add((host, port))
+                    addr_block.append(' ')
+                    addr_block.append('ipv6only=on')
+                addr_blocks.append(addr_block)
+            else:
+                tuple_string = f'{host}:{port}' if host else f'{port}'
+                addr_block = ['\n    ',
+                              'listen',
+                              ' ',
+                              tuple_string,
+                              ' ',
+                              'ssl']
+                addr_blocks.append(addr_block)
 
         snakeoil_cert, snakeoil_key = self._get_snakeoil_paths()
 
         ssl_block = ([
-            ipv6_block,
-            ipv4_block,
+            *addr_blocks,
             ['\n    ', 'ssl_certificate', ' ', snakeoil_cert],
             ['\n    ', 'ssl_certificate_key', ' ', snakeoil_key],
             ['\n    ', 'include', ' ', self.mod_ssl_conf],
@@ -761,12 +810,12 @@ class NginxConfigurator(common.Configurator):
     ##################################
     # enhancement methods (Installer)
     ##################################
-    def supported_enhancements(self) -> List[str]:
+    def supported_enhancements(self) -> list[str]:
         """Returns currently supported enhancements."""
         return ['redirect', 'ensure-http-header', 'staple-ocsp']
 
     def enhance(self, domain: str, enhancement: str,
-                options: Optional[Union[str, List[str]]] = None) -> None:
+                options: Optional[Union[str, list[str]]] = None) -> None:
         """Enhance configuration.
 
         :param str domain: domain to enhance
@@ -787,7 +836,7 @@ class NginxConfigurator(common.Configurator):
         test_redirect_block = _test_block_from_block(_redirect_block_for_domain(domain))
         return vhost.contains_list(test_redirect_block)
 
-    def _set_http_header(self, domain: str, header_substring: Union[str, List[str], None]) -> None:
+    def _set_http_header(self, domain: str, header_substring: Union[str, list[str], None]) -> None:
         """Enables header identified by header_substring on domain.
 
         If the vhost is listening plaintextishly, separates out the relevant
@@ -836,8 +885,8 @@ class NginxConfigurator(common.Configurator):
         self.parser.add_server_directives(
             vhost, redirect_block, insert_at_top=True)
 
-    def _split_block(self, vhost: obj.VirtualHost, only_directives: Optional[List[str]] = None
-                     ) -> Tuple[obj.VirtualHost, obj.VirtualHost]:
+    def _split_block(self, vhost: obj.VirtualHost, only_directives: Optional[list[str]] = None
+                     ) -> tuple[obj.VirtualHost, obj.VirtualHost]:
         """Splits this "virtual host" (i.e. this nginx server block) into
         separate HTTP and HTTPS blocks.
 
@@ -871,7 +920,7 @@ class NginxConfigurator(common.Configurator):
         return http_vhost, vhost
 
     def _enable_redirect(self, domain: str,
-                         unused_options: Optional[Union[str, List[str]]]) -> None:
+                         unused_options: Optional[Union[str, list[str]]]) -> None:
         """Redirect all equivalent HTTP traffic to ssl_vhost.
 
         If the vhost is listening plaintextishly, separate out the
@@ -928,7 +977,7 @@ class NginxConfigurator(common.Configurator):
                 self.DEFAULT_LISTEN_PORT, vhost.filep)
 
     def _enable_ocsp_stapling(self, domain: str,
-                              chain_path: Optional[Union[str, List[str]]]) -> None:
+                              chain_path: Optional[Union[str, list[str]]]) -> None:
         """Include OCSP response in TLS handshake
 
         :param str domain: domain to enable OCSP response for
@@ -1027,7 +1076,7 @@ class NginxConfigurator(common.Configurator):
                 "Unable to run %s -V" % self.conf('ctl'))
         return text
 
-    def get_version(self) -> Tuple[int, ...]:
+    def get_version(self) -> tuple[int, ...]:
         """Return version of Nginx Server.
 
         Version is returned as tuple. (ie. 2.4.7 = (2, 4, 7))
@@ -1085,7 +1134,7 @@ class NginxConfigurator(common.Configurator):
         """
         text = self._nginx_version()
 
-        matches = re.findall(r"running with OpenSSL ([^ ]+) ", text)
+        matches: list[str] = re.findall(r"running with OpenSSL ([^ ]+) ", text)
         if not matches:
             matches = re.findall(r"built with OpenSSL ([^ ]+) ", text)
             if not matches:
@@ -1177,13 +1226,13 @@ class NginxConfigurator(common.Configurator):
     ###########################################################################
     # Challenges Section for Authenticator
     ###########################################################################
-    def get_chall_pref(self, unused_domain: str) -> List[Type[challenges.Challenge]]:
+    def get_chall_pref(self, unused_domain: str) -> list[type[challenges.Challenge]]:
         """Return list of challenge preferences."""
         return [challenges.HTTP01]
 
     # Entry point in main.py for performing challenges
-    def perform(self, achalls: List[achallenges.AnnotatedChallenge]
-                ) -> List[challenges.ChallengeResponse]:
+    def perform(self, achalls: list[achallenges.AnnotatedChallenge]
+                ) -> list[challenges.ChallengeResponse]:
         """Perform the configuration related challenge.
 
         This function currently assumes all challenges will be fulfilled.
@@ -1192,7 +1241,7 @@ class NginxConfigurator(common.Configurator):
 
         """
         self._chall_out += len(achalls)
-        responses: List[Optional[challenges.ChallengeResponse]] = [None] * len(achalls)
+        responses: list[Optional[challenges.ChallengeResponse]] = [None] * len(achalls)
         http_doer = http_01.NginxHttp01(self)
 
         for i, achall in enumerate(achalls):
@@ -1218,7 +1267,7 @@ class NginxConfigurator(common.Configurator):
         return [response for response in responses if response]
 
     # called after challenges are performed
-    def cleanup(self, achalls: List[achallenges.AnnotatedChallenge]) -> None:
+    def cleanup(self, achalls: list[achallenges.AnnotatedChallenge]) -> None:
         """Revert all challenges."""
         self._chall_out -= len(achalls)
 
@@ -1228,13 +1277,13 @@ class NginxConfigurator(common.Configurator):
             self.restart()
 
 
-def _test_block_from_block(block: List[Any]) -> List[Any]:
+def _test_block_from_block(block: list[Any]) -> list[Any]:
     test_block = nginxparser.UnspacedList(block)
     parser.comment_directive(test_block, 0)
     return test_block[:-1]
 
 
-def _redirect_block_for_domain(domain: str) -> List[Any]:
+def _redirect_block_for_domain(domain: str) -> list[Any]:
     updated_domain = domain
     match_symbol = '='
     if util.is_wildcard_domain(domain):
@@ -1263,7 +1312,7 @@ def nginx_restart(nginx_ctl: str, nginx_conf: str, sleep_duration: int) -> None:
 
     """
     try:
-        reload_output: Text = ""
+        reload_output: str = ""
         with tempfile.TemporaryFile() as out:
             proc = subprocess.run([nginx_ctl, "-c", nginx_conf, "-s", "reload"],
                                   env=util.env_no_snap_for_external_calls(),
